@@ -1,14 +1,15 @@
-# Copyright 1995 by Andreas Koenig (see AUTHOR below)
+# Copyright 1994-1996 by Andreas Knig (see AUTHOR below)
+# This is free software.
 # Permission is granted to use and copy this piece of code under the
-# same conditions as perl itself.
+# same terms as perl itself.
 
 # Major hacks by tchrist.
 
 package Devel::Symdump;
 
-use 5.003;
+BEGIN {require 5.003;}
 use strict;
-use vars qw($Defaults $VERSION);
+use vars qw($Defaults $VERSION $ENTRY @ENTRY %ENTRY *ENTRY %stab);
 
 $Defaults = {
     'RECURS' => 0,
@@ -24,10 +25,9 @@ $Defaults = {
 	}
 };
 
-$VERSION = '1.22'; #avoid warning
+$VERSION = '1.23';
 
-# Revision is not linked to $VERSION anymore
-# $Id: Symdump.pm,v 1.29 1996/12/02 01:11:51 k Exp $
+# $Id: Symdump.pm,v 1.32 1996/12/02 11:16:30 k Exp $
 
 $Defaults = $Defaults;
 
@@ -51,60 +51,60 @@ sub new {
 
 sub _doit {
     my($self,@packages) = @_;
-    # The following line should be
-    # @packages = ("main") unless defined(@packages);
-    # but currently works around a bug in perl5.001 with defined() in reentered scope 
-    @packages = ("main") unless defined(@packages) && @packages>0;
+    @packages = ("main") unless @packages;
     $self->{RESULT} = $self->_symdump(@packages);
     return $self;
 }
 
 sub _symdump {
     my($self,@packages) = @_ ;
-    my($key,$val,$num,$pack,@todo,$tmp);
+    my($key,$val,$num,$pack,$package,@todo,$tmp);
     my $result = {};
-    foreach $pack (@packages){
+    foreach $package (@packages){
+	$pack = $package;
 	no strict;
-#	local(*stab) = *{"$pack\::"};
-	while (($key,$val) = each(%{*{"$pack\::"}})) {
+	local(*stab) = *{"$package\:\:"};
+	while (($key,$val) = each(%stab)) {
+	    local(*ENTRY) = $val;
 	    my $gotone = 0;
 	    #### SCALAR ####
-	    if (defined $val && defined *{$val}{SCALAR}) {
+	    if (defined $ENTRY) {
 		$result->{$pack}{SCALARS}{$key}++;
 		$gotone++;
 	    }
 	    #### ARRAY ####
-	    if (defined $val && defined *{$val}{ARRAY}) {
+	    if (defined @ENTRY) {
 		$result->{$pack}{ARRAYS}{$key}++;
 		$gotone++;
 	    }
 	    #### HASH ####
-	    if (defined $val && defined *{$val}{HASH} && $key !~ /::/) {
+	    if (defined %ENTRY && $key !~ /::/) {
 		$result->{$pack}{HASHES}{$key}++;
 		$gotone++;
 	    }
 	    #### PACKAGE ####
-	    if (defined $val && defined *{$val}{HASH} && $key =~ /::$/ &&
-		    $key ne "main::" )
+	    if (defined %ENTRY && $key =~ /::/ &&
+		    $key ne "main::" && ($tmp = "$pack\:\:") &&
+		    $tmp !~ /^$key/ )
 	    {
-		my($p) = $pack ne "main" ? "$pack\::" : "";
+		my($p) = $pack ne "main" ? "$pack\:\:" : "";
 		($p .= $key) =~ s/::$//;
 		$result->{$pack}{PACKAGES}{$p}++;
 		$gotone++;
 		push @todo, $p;
 	    }
 	    #### FUNCTION ####
-	    if (defined $val && defined *{$val}{CODE}) {
+	    if (defined &ENTRY) {
 		$result->{$pack}{FUNCTIONS}{$key}++;
 		$gotone++;
 	    }
 	    #### FILEHANDLE ####
-	    if (defined $val && defined fileno(*{$val}{FILEHANDLE})){
+	    if (defined fileno(ENTRY)){
 		$result->{$pack}{FILEHANDLES}{$key}++;
 		$gotone++;
 	    }
 	    #### DIRHANDLE ####
-	    if (defined $val && defined telldir(*{$val})){
+	    if (defined telldir(ENTRY)){
 		$result->{$pack}{DIRHANDLES}{$key}++;
 		$gotone++;
 	    }
@@ -194,10 +194,7 @@ sub diff {
 AUTOLOAD {
     my($self,@packages) = @_;
     unless (ref $self) {
-#	$self = $self->new(@packages); # perl5.001m might have a bug that doesn't allow this (?)
-	my $tself;
-	$tself = $self->new(@packages);
-	$self = $tself;
+	$self = $self->new(@packages);
     }
     no strict "vars";
     (my $auto = $AUTOLOAD) =~ s/.*:://;
@@ -237,6 +234,7 @@ Devel::Symdump - dump symbol names or the symbol table
     @array = $obj->unknowns;
 	
     $string = $obj->as_string;
+    $string = $obj->as_HTML;
     $string = $obj1->diff($obj2);
 
     # Methods with autogenerated objects
@@ -274,7 +272,8 @@ C<@> or C<%>.  In a scalar context, they will return the number of
 such symbols.  Unknown symbols are usually either formats or variables
 that haven't yet got a defined value.
 
-As_string() prints a simple string representation of the object.
+As_string() and as_HTML() print simple string/HTML representations of
+the object.
 
 Diff() prints the difference between two Devel::Symdump objects in
 human readable form. The format is similar to the one used by the
@@ -295,6 +294,6 @@ F<dumpvar.pl> by Larry Wall.
 
 =head1 VERSION
 
-This release is $Revision: 1.29 $.
+This release is $Revision: 1.32 $.
 
 =cut
